#ifndef __EDG_WORKLOAD_LOGGING_CLIENT_JOBSTAT_H__
#define __EDG_WORKLOAD_LOGGING_CLIENT_JOBSTAT_H__

/* 

  !! Automatically generated file
  !! Do not edit, your changes will be discarded upon build
  !! Change the corresponding template file ./logging/client/jobstat.h.T

*/

/*!
 * \file client/jobstat.h  
 * \brief edg_wll_JobStat definition and related stuff
 */

#ident "$Header: /local/reps/lcgware/workload/logging/client/jobstat.h.T,v 1.3 2003/04/10 14:05:58 salvet Exp $"

#ifdef __cplusplus
extern "C" {
#endif

/**
 * Miscelaneous job status numeric codes
 */

enum edg_wll_StatJobtype {
	EDG_WLL_STAT_SIMPLE,	/**< simple job */
	EDG_WLL_STAT_DAG,	/**< composite job */
};
enum edg_wll_StatDone_code {
	EDG_WLL_STAT_OK,	/**< Finished correctly */
	EDG_WLL_STAT_FAILED,	/**< Execution failed */
	EDG_WLL_STAT_CANCELLED,	/**< Cancelled by user */
};


/*!
 *
 * Job status numeric code
 */

typedef enum _edg_wll_JobStatCode {
/** Indicates invalid edg_wll_JobStat structure */
	EDG_WLL_JOB_UNDEF = 0,
	EDG_WLL_JOB_SUBMITTED,	/**< entered by the user to the User Interface or registered by Job Partitioner */
	EDG_WLL_JOB_WAITING,	/**< Accepted by WMS, waiting for resource allocation */
	EDG_WLL_JOB_READY,	/**< Matching resources found */
	EDG_WLL_JOB_SCHEDULED,	/**< Accepted by LRMS queue */
	EDG_WLL_JOB_RUNNING,	/**< Executable is running */
	EDG_WLL_JOB_DONE,	/**< Execution finished, output is available */
	EDG_WLL_JOB_CLEARED,	/**< Output transfered back to user and freed */
	EDG_WLL_JOB_ABORTED,	/**< Aborted by system (at any stage) */
	EDG_WLL_JOB_CANCELLED,	/**< Cancelled by user */
	EDG_WLL_JOB_UNKNOWN,	/**< Status cannot be determined */
	EDG_WLL_JOB_PURGED,	/**< Job has been purged from bookkeeping server (for LB->RGMA interface) */
	EDG_WLL_NUMBER_OF_STATCODES /**< Number of meaningful status codes */
} edg_wll_JobStatCode;

/*!
 *
 * Pair tag = value.
 */
typedef struct _edg_wll_TagValue {
        char *  tag;    /**< User-specified information tag */
        char *  value;  /**< Value assigned to user-specified information tag */
} edg_wll_TagValue;


/*!
 *
 * Description of the job status.
 * Returned by the edg_wll_JobStatus() function
 */
typedef struct _edg_wll_JobStat {
	edg_wll_JobStatCode	state;	        /**< status code */
	edg_wlc_JobId	jobId;	/**< Id of the job */
	char *	owner;	/**< Job owner */
	enum edg_wll_StatJobtype	jobtype;	/**< Type of job */
	edg_wlc_JobId	parent_job;	/**< parent job of subjob */
	char *	seed;	/**< string used for generation of subjob IDs */
	int	children_num;	/**< number of subjobs */
	char **	children;	/**< list of subjob IDs */
	int *	children_hist;	/**< summary (histogram) of children job states */
	struct _edg_wll_JobStat *	children_states;	/**< full status information of the children */
	char *	condorId;	/**< Id within Condor-G */
	char *	globusId;	/**< Globus allocated Id */
	char *	localId;	/**< Id within LRMS */
	char *	jdl;	/**< User submitted job description */
	char *	matched_jdl;	/**< Full job description after matchmaking */
	char *	destination;	/**< ID of CE where the job is being sent */
	char *	condor_jdl;	/**< ClassAd passed to Condor-G for last job execution */
	char *	rsl;	/**< Job RSL sent to Globus */
	char *	reason;	/**< Reason of being in this status, if any */
	char *	location;	/**< Where the job is being processed */
	char *	ce_node;	/**< Worker node where the job is executed */
	char *	network_server;	/**< Network server handling the job */
	int	subjob_failed;	/**< Subjob failed (the parent job will fail too) */
	enum edg_wll_StatDone_code	done_code;	/**< Return code */
	int	exit_code;	/**< Unix exit code */
	int	resubmitted;	/**< The job was resubmitted */
	int	cancelling;	/**< Cancellation request in progress */
	char *	cancelReason;	/**< Reason of cancel */
	int	cpuTime;	/**< Consumed CPU time */
	edg_wll_TagValue *	user_tags;	/**< List of pairs (user_tag, user_value) */
	struct timeval	stateEnterTime;	/**< When entered this status */
	struct timeval	lastUpdateTime;	/**< Last known event of the job */
	int *	stateEnterTimes;	/**< When all previous states were entered */
	int	expectUpdate;	/**< Some logged information has not arrived yet */
	char *	expectFrom;	/**< Sources of the missing information */
	char *	acl;	/**< ACL of the job */

} edg_wll_JobStat;

/**
 * \name edg_wll_JobStat manipulation
 */

/*@{*/

/**
 * Initialize empty status structure.
 * Fills in the stucture with NULL's or values with no meaning
 */
 
extern int edg_wll_InitStatus(edg_wll_JobStat *);

/**
 * Initialize dest structure and copy source status to this destination
 */
 
extern edg_wll_JobStat *edg_wll_CpyStatus(edg_wll_JobStat *,edg_wll_JobStat *);

/** 
 * Free status structure contents.
 * \warning The structure itself is not freed.
 */
extern void edg_wll_FreeStatus(edg_wll_JobStat *);

/**
 * Convert string job status to numeric code.
 */
extern edg_wll_JobStatCode edg_wll_StringToStat(const char *);

/** 
 * Convert numeric job status code to string representation
 */
extern char *edg_wll_StatToString(edg_wll_JobStatCode);

/*@}*/

#ifdef __cplusplus
}
#endif

#endif /* __EDG_WORKLOAD_LOGGING_CLIENT_JOBSTAT_H__ */
