#ifndef __EDG_WORKLOAD_LOGGING_CLIENT_PRODUCER_H__
#define __EDG_WORKLOAD_LOGGING_CLIENT_PRODUCER_H__

/**
 * \file edg/workload/logging/client/producer.h
 * \brief client API for storing data into L&B service
 */

#ident "$Header: /local/reps/lcgware/workload/logging/client/producer.h.T,v 1.35 2004/01/14 15:47:54 ljocha Exp $"
/*

  !! Automatically generated file
  !! Do not edit, your changes will be discarded upon build
  !! Change the corresponding template file ./logging/client/producer.h.T

*/

#ifdef __cplusplus
extern "C" {
#endif

#include "edg/workload/logging/client/context.h"
#include "edg/workload/logging/client/events.h"
#include "edg/workload/logging/common/authz.h"

/* Event sources: */

#if 0 /* obsolete */
#define EDG_WLL_SOURCE_UI           "UserInterface"
#define EDG_WLL_SOURCE_RB           "ResourceBroker"
#define EDG_WLL_SOURCE_JSS          "JobSubmissionService"  /* aka Condor-G */
#define EDG_WLL_SOURCE_JOBMGR       "GlobusJobmanager"
#define EDG_WLL_SOURCE_LRMS         "LocalResourceManager"
#define EDG_WLL_SOURCE_APP          "Application"

#define EDG_WLL_SOURCE_NS	"NetworkServer"
#define EDG_WLL_SOURCE_WM	"WorkloadManager"
#define EDG_WLL_SOURCE_BH	"BigHelper"
#define EDG_WLL_SOURCE_LM	"LogMonitor"

#endif

/* Event formats: */

#define EDG_WLL_FORMAT_COMMON	"DATE=%s HOST=\"%|Us\" PROG=edg-wms LVL=%s DG.PRIORITY=%d DG.SOURCE=\"%|Us\" DG.SRC_INSTANCE=\"%|Us\" DG.EVNT=\"%s\" DG.JOBID=\"%s\" DG.SEQCODE=\"%|Us\" "
#define EDG_WLL_FORMAT_USER	"DG.USER=\"%|Us\" "
#define EDG_WLL_FORMAT_TRANSFER	"DG.TRANSFER.DESTINATION=\"%s\" DG.TRANSFER.DEST_HOST=\"%|Us\" DG.TRANSFER.DEST_INSTANCE=\"%|Us\" DG.TRANSFER.JOB=\"%|Us\" DG.TRANSFER.RESULT=\"%s\" DG.TRANSFER.REASON=\"%|Us\" DG.TRANSFER.DEST_JOBID=\"%|Us\" "
#define EDG_WLL_FORMAT_ACCEPTED	"DG.ACCEPTED.FROM=\"%s\" DG.ACCEPTED.FROM_HOST=\"%|Us\" DG.ACCEPTED.FROM_INSTANCE=\"%|Us\" DG.ACCEPTED.LOCAL_JOBID=\"%|Us\" "
#define EDG_WLL_FORMAT_REFUSED	"DG.REFUSED.FROM=\"%s\" DG.REFUSED.FROM_HOST=\"%|Us\" DG.REFUSED.FROM_INSTANCE=\"%|Us\" DG.REFUSED.REASON=\"%|Us\" "
#define EDG_WLL_FORMAT_ENQUEUED	"DG.ENQUEUED.QUEUE=\"%|Us\" DG.ENQUEUED.JOB=\"%|Us\" DG.ENQUEUED.RESULT=\"%s\" DG.ENQUEUED.REASON=\"%|Us\" "
#define EDG_WLL_FORMAT_DEQUEUED	"DG.DEQUEUED.QUEUE=\"%|Us\" DG.DEQUEUED.LOCAL_JOBID=\"%|Us\" "
#define EDG_WLL_FORMAT_HELPERCALL	"DG.HELPERCALL.HELPER_NAME=\"%|Us\" DG.HELPERCALL.HELPER_PARAMS=\"%|Us\" DG.HELPERCALL.SRC_ROLE=\"%s\" "
#define EDG_WLL_FORMAT_HELPERRETURN	"DG.HELPERRETURN.HELPER_NAME=\"%|Us\" DG.HELPERRETURN.RETVAL=\"%|Us\" DG.HELPERRETURN.SRC_ROLE=\"%s\" "
#define EDG_WLL_FORMAT_RUNNING	"DG.RUNNING.NODE=\"%|Us\" "
#define EDG_WLL_FORMAT_RESUBMISSION	"DG.RESUBMISSION.RESULT=\"%s\" DG.RESUBMISSION.REASON=\"%|Us\" DG.RESUBMISSION.TAG=\"%|Us\" "
#define EDG_WLL_FORMAT_DONE	"DG.DONE.STATUS_CODE=\"%s\" DG.DONE.REASON=\"%|Us\" DG.DONE.EXIT_CODE=\"%d\" "
#define EDG_WLL_FORMAT_CANCEL	"DG.CANCEL.STATUS_CODE=\"%s\" DG.CANCEL.REASON=\"%|Us\" "
#define EDG_WLL_FORMAT_ABORT	"DG.ABORT.REASON=\"%|Us\" "
#define EDG_WLL_FORMAT_CLEAR	"DG.CLEAR.REASON=\"%s\" "
#define EDG_WLL_FORMAT_PURGE	""
#define EDG_WLL_FORMAT_MATCH	"DG.MATCH.DEST_ID=\"%|Us\" "
#define EDG_WLL_FORMAT_PENDING	"DG.PENDING.REASON=\"%|Us\" "
#define EDG_WLL_FORMAT_REGJOB	"DG.REGJOB.JDL=\"%|Us\" DG.REGJOB.NS=\"%|Us\" DG.REGJOB.PARENT=\"%s\" DG.REGJOB.JOBTYPE=\"%s\" DG.REGJOB.NSUBJOBS=\"%d\" DG.REGJOB.SEED=\"%|Us\" "
#define EDG_WLL_FORMAT_CHKPT	"DG.CHKPT.TAG=\"%|Us\" DG.CHKPT.CLASSAD=\"%|Us\" "
#define EDG_WLL_FORMAT_LISTENER	"DG.LISTENER.SVC_NAME=\"%|Us\" DG.LISTENER.SVC_HOST=\"%|Us\" DG.LISTENER.SVC_PORT=\"%d\" "
#define EDG_WLL_FORMAT_CURDESCR	"DG.CURDESCR.DESCR=\"%|Us\" "
#define EDG_WLL_FORMAT_USERTAG	"DG.USERTAG.NAME=\"%|Us\" DG.USERTAG.VALUE=\"%|Us\" "
#define EDG_WLL_FORMAT_CHANGEACL	"DG.CHANGEACL.USER_ID=\"%|Us\" DG.CHANGEACL.USER_ID_TYPE=\"%d\" DG.CHANGEACL.PERMISSION=\"%d\" DG.CHANGEACL.PERMISSION_TYPE=\"%d\" DG.CHANGEACL.OPERATION=\"%d\" "
#define EDG_WLL_FORMAT_SYSCMPSTAT	"DG.SCHED.STATUS=\"%|Us\" "
#define EDG_WLL_FORMAT_SYSCLSTAT	"DG.SCHED.NODE=\"%|Us\" DG.SCHED.STATUS=\"%|Us\" "


/* edg_wll_LogEvent shortcuts */

/**
 * \fn int edg_wll_LogTransfer(edg_wll_Context context, const edg_wll_Source destination, const char * dest_host, const char * dest_instance, const char * job, const char * result, const char * reason, const char * dest_jobid); 
 * \brief simple wrapper around edg_wll_LogEvent for event Transfer
 * \param context	context to work with,
 * \param destination	destination where the job is being transfered to
 * \param dest_host	destination hostname
 * \param dest_instance	destination instance
 * \param job	job description in receiver language
 * \param result	result of the attempt
 * \param reason	detailed description of transfer, especially reason of failure
 * \param dest_jobid	destination internal jobid
 * \see edg_wll_LogEvent()
 */

extern int edg_wll_LogTransfer(edg_wll_Context context, const edg_wll_Source destination, const char * dest_host, const char * dest_instance, const char * job, const char * result, const char * reason, const char * dest_jobid);


/**
 * \fn int edg_wll_LogTransferSTART(edg_wll_Context context, const edg_wll_Source destination, const char * dest_host, const char * dest_instance, const char * job, const char * reason, const char * dest_jobid); 
 * \brief simple wrapper around edg_wll_LogEvent for event Transfer, result START
 * \param context	context to work with,
 * \param destination	destination where the job is being transfered to
 * \param dest_host	destination hostname
 * \param dest_instance	destination instance
 * \param job	job description in receiver language
 * \param reason	detailed description of transfer, especially reason of failure
 * \param dest_jobid	destination internal jobid
 * \see edg_wll_LogEvent()
 */

extern int edg_wll_LogTransferSTART(edg_wll_Context context, const edg_wll_Source destination, const char * dest_host, const char * dest_instance, const char * job, const char * reason, const char * dest_jobid);


/**
 * \fn int edg_wll_LogTransferOK(edg_wll_Context context, const edg_wll_Source destination, const char * dest_host, const char * dest_instance, const char * job, const char * reason, const char * dest_jobid); 
 * \brief simple wrapper around edg_wll_LogEvent for event Transfer, result OK
 * \param context	context to work with,
 * \param destination	destination where the job is being transfered to
 * \param dest_host	destination hostname
 * \param dest_instance	destination instance
 * \param job	job description in receiver language
 * \param reason	detailed description of transfer, especially reason of failure
 * \param dest_jobid	destination internal jobid
 * \see edg_wll_LogEvent()
 */

extern int edg_wll_LogTransferOK(edg_wll_Context context, const edg_wll_Source destination, const char * dest_host, const char * dest_instance, const char * job, const char * reason, const char * dest_jobid);


/**
 * \fn int edg_wll_LogTransferREFUSED(edg_wll_Context context, const edg_wll_Source destination, const char * dest_host, const char * dest_instance, const char * job, const char * reason, const char * dest_jobid); 
 * \brief simple wrapper around edg_wll_LogEvent for event Transfer, result REFUSED
 * \param context	context to work with,
 * \param destination	destination where the job is being transfered to
 * \param dest_host	destination hostname
 * \param dest_instance	destination instance
 * \param job	job description in receiver language
 * \param reason	detailed description of transfer, especially reason of failure
 * \param dest_jobid	destination internal jobid
 * \see edg_wll_LogEvent()
 */

extern int edg_wll_LogTransferREFUSED(edg_wll_Context context, const edg_wll_Source destination, const char * dest_host, const char * dest_instance, const char * job, const char * reason, const char * dest_jobid);


/**
 * \fn int edg_wll_LogTransferFAIL(edg_wll_Context context, const edg_wll_Source destination, const char * dest_host, const char * dest_instance, const char * job, const char * reason, const char * dest_jobid); 
 * \brief simple wrapper around edg_wll_LogEvent for event Transfer, result FAIL
 * \param context	context to work with,
 * \param destination	destination where the job is being transfered to
 * \param dest_host	destination hostname
 * \param dest_instance	destination instance
 * \param job	job description in receiver language
 * \param reason	detailed description of transfer, especially reason of failure
 * \param dest_jobid	destination internal jobid
 * \see edg_wll_LogEvent()
 */

extern int edg_wll_LogTransferFAIL(edg_wll_Context context, const edg_wll_Source destination, const char * dest_host, const char * dest_instance, const char * job, const char * reason, const char * dest_jobid);


/**
 * \fn int edg_wll_LogAccepted(edg_wll_Context context, const edg_wll_Source from, const char * from_host, const char * from_instance, const char * local_jobid); 
 * \brief simple wrapper around edg_wll_LogEvent for event Accepted
 * \param context	context to work with,
 * \param from	where was the job received from
 * \param from_host	sending component hostname
 * \param from_instance	sending component instance
 * \param local_jobid	new jobId (Condor, Globus ...) assigned by the receiving component
 * \see edg_wll_LogEvent()
 */

extern int edg_wll_LogAccepted(edg_wll_Context context, const edg_wll_Source from, const char * from_host, const char * from_instance, const char * local_jobid);


/**
 * \fn int edg_wll_LogRefused(edg_wll_Context context, const edg_wll_Source from, const char * from_host, const char * from_instance, const char * reason); 
 * \brief simple wrapper around edg_wll_LogEvent for event Refused
 * \param context	context to work with,
 * \param from	where was the job received from
 * \param from_host	sending component hostname
 * \param from_instance	sending component instance
 * \param reason	reason of refusal
 * \see edg_wll_LogEvent()
 */

extern int edg_wll_LogRefused(edg_wll_Context context, const edg_wll_Source from, const char * from_host, const char * from_instance, const char * reason);


/**
 * \fn int edg_wll_LogEnQueued(edg_wll_Context context, const char * queue, const char * job, const char * result, const char * reason); 
 * \brief simple wrapper around edg_wll_LogEvent for event EnQueued
 * \param context	context to work with,
 * \param queue	destination queue
 * \param job	job description in receiver language
 * \param result	result of the attempt
 * \param reason	detailed description of transfer, especially reason of failure
 * \see edg_wll_LogEvent()
 */

extern int edg_wll_LogEnQueued(edg_wll_Context context, const char * queue, const char * job, const char * result, const char * reason);


/**
 * \fn int edg_wll_LogEnQueuedSTART(edg_wll_Context context, const char * queue, const char * job, const char * reason); 
 * \brief simple wrapper around edg_wll_LogEvent for event EnQueued, result START
 * \param context	context to work with,
 * \param queue	destination queue
 * \param job	job description in receiver language
 * \param reason	detailed description of transfer, especially reason of failure
 * \see edg_wll_LogEvent()
 */

extern int edg_wll_LogEnQueuedSTART(edg_wll_Context context, const char * queue, const char * job, const char * reason);


/**
 * \fn int edg_wll_LogEnQueuedOK(edg_wll_Context context, const char * queue, const char * job, const char * reason); 
 * \brief simple wrapper around edg_wll_LogEvent for event EnQueued, result OK
 * \param context	context to work with,
 * \param queue	destination queue
 * \param job	job description in receiver language
 * \param reason	detailed description of transfer, especially reason of failure
 * \see edg_wll_LogEvent()
 */

extern int edg_wll_LogEnQueuedOK(edg_wll_Context context, const char * queue, const char * job, const char * reason);


/**
 * \fn int edg_wll_LogEnQueuedREFUSED(edg_wll_Context context, const char * queue, const char * job, const char * reason); 
 * \brief simple wrapper around edg_wll_LogEvent for event EnQueued, result REFUSED
 * \param context	context to work with,
 * \param queue	destination queue
 * \param job	job description in receiver language
 * \param reason	detailed description of transfer, especially reason of failure
 * \see edg_wll_LogEvent()
 */

extern int edg_wll_LogEnQueuedREFUSED(edg_wll_Context context, const char * queue, const char * job, const char * reason);


/**
 * \fn int edg_wll_LogEnQueuedFAIL(edg_wll_Context context, const char * queue, const char * job, const char * reason); 
 * \brief simple wrapper around edg_wll_LogEvent for event EnQueued, result FAIL
 * \param context	context to work with,
 * \param queue	destination queue
 * \param job	job description in receiver language
 * \param reason	detailed description of transfer, especially reason of failure
 * \see edg_wll_LogEvent()
 */

extern int edg_wll_LogEnQueuedFAIL(edg_wll_Context context, const char * queue, const char * job, const char * reason);


/**
 * \fn int edg_wll_LogDeQueued(edg_wll_Context context, const char * queue, const char * local_jobid); 
 * \brief simple wrapper around edg_wll_LogEvent for event DeQueued
 * \param context	context to work with,
 * \param queue	queue name
 * \param local_jobid	new jobId assigned by the receiving component
 * \see edg_wll_LogEvent()
 */

extern int edg_wll_LogDeQueued(edg_wll_Context context, const char * queue, const char * local_jobid);


/**
 * \fn int edg_wll_LogHelperCall(edg_wll_Context context, const char * helper_name, const char * helper_params, const char * src_role); 
 * \brief simple wrapper around edg_wll_LogEvent for event HelperCall
 * \param context	context to work with,
 * \param helper_name	name of the called component
 * \param helper_params	parameters of the call
 * \param src_role	whether the logging component is called or calling one
 * \see edg_wll_LogEvent()
 */

extern int edg_wll_LogHelperCall(edg_wll_Context context, const char * helper_name, const char * helper_params, const char * src_role);


/**
 * \fn int edg_wll_LogHelperCallCALLING(edg_wll_Context context, const char * helper_name, const char * helper_params); 
 * \brief simple wrapper around edg_wll_LogEvent for event HelperCall, src_role CALLING
 * \param context	context to work with,
 * \param helper_name	name of the called component
 * \param helper_params	parameters of the call
 * \see edg_wll_LogEvent()
 */

extern int edg_wll_LogHelperCallCALLING(edg_wll_Context context, const char * helper_name, const char * helper_params);


/**
 * \fn int edg_wll_LogHelperCallCALLED(edg_wll_Context context, const char * helper_name, const char * helper_params); 
 * \brief simple wrapper around edg_wll_LogEvent for event HelperCall, src_role CALLED
 * \param context	context to work with,
 * \param helper_name	name of the called component
 * \param helper_params	parameters of the call
 * \see edg_wll_LogEvent()
 */

extern int edg_wll_LogHelperCallCALLED(edg_wll_Context context, const char * helper_name, const char * helper_params);


/**
 * \fn int edg_wll_LogHelperReturn(edg_wll_Context context, const char * helper_name, const char * retval, const char * src_role); 
 * \brief simple wrapper around edg_wll_LogEvent for event HelperReturn
 * \param context	context to work with,
 * \param helper_name	name of the called component
 * \param retval	returned data
 * \param src_role	whether the logging component is called or calling one
 * \see edg_wll_LogEvent()
 */

extern int edg_wll_LogHelperReturn(edg_wll_Context context, const char * helper_name, const char * retval, const char * src_role);


/**
 * \fn int edg_wll_LogHelperReturnCALLING(edg_wll_Context context, const char * helper_name, const char * retval); 
 * \brief simple wrapper around edg_wll_LogEvent for event HelperReturn, src_role CALLING
 * \param context	context to work with,
 * \param helper_name	name of the called component
 * \param retval	returned data
 * \see edg_wll_LogEvent()
 */

extern int edg_wll_LogHelperReturnCALLING(edg_wll_Context context, const char * helper_name, const char * retval);


/**
 * \fn int edg_wll_LogHelperReturnCALLED(edg_wll_Context context, const char * helper_name, const char * retval); 
 * \brief simple wrapper around edg_wll_LogEvent for event HelperReturn, src_role CALLED
 * \param context	context to work with,
 * \param helper_name	name of the called component
 * \param retval	returned data
 * \see edg_wll_LogEvent()
 */

extern int edg_wll_LogHelperReturnCALLED(edg_wll_Context context, const char * helper_name, const char * retval);


/**
 * \fn int edg_wll_LogRunning(edg_wll_Context context, const char * node); 
 * \brief simple wrapper around edg_wll_LogEvent for event Running
 * \param context	context to work with,
 * \param node	worker node where the executable is run
 * \see edg_wll_LogEvent()
 */

extern int edg_wll_LogRunning(edg_wll_Context context, const char * node);


/**
 * \fn int edg_wll_LogResubmission(edg_wll_Context context, const char * result, const char * reason, const char * tag); 
 * \brief simple wrapper around edg_wll_LogEvent for event Resubmission
 * \param context	context to work with,
 * \param result	result code
 * \param reason	reason for the decision
 * \param tag	value of the attribute on which the decision is based
 * \see edg_wll_LogEvent()
 */

extern int edg_wll_LogResubmission(edg_wll_Context context, const char * result, const char * reason, const char * tag);


/**
 * \fn int edg_wll_LogResubmissionWILLRESUB(edg_wll_Context context, const char * reason, const char * tag); 
 * \brief simple wrapper around edg_wll_LogEvent for event Resubmission, result WILLRESUB
 * \param context	context to work with,
 * \param reason	reason for the decision
 * \param tag	value of the attribute on which the decision is based
 * \see edg_wll_LogEvent()
 */

extern int edg_wll_LogResubmissionWILLRESUB(edg_wll_Context context, const char * reason, const char * tag);


/**
 * \fn int edg_wll_LogResubmissionWONTRESUB(edg_wll_Context context, const char * reason, const char * tag); 
 * \brief simple wrapper around edg_wll_LogEvent for event Resubmission, result WONTRESUB
 * \param context	context to work with,
 * \param reason	reason for the decision
 * \param tag	value of the attribute on which the decision is based
 * \see edg_wll_LogEvent()
 */

extern int edg_wll_LogResubmissionWONTRESUB(edg_wll_Context context, const char * reason, const char * tag);


/**
 * \fn int edg_wll_LogDone(edg_wll_Context context, const char * status_code, const char * reason, const int exit_code); 
 * \brief simple wrapper around edg_wll_LogEvent for event Done
 * \param context	context to work with,
 * \param status_code	way of termination
 * \param reason	reason for the change
 * \param exit_code	process exit code
 * \see edg_wll_LogEvent()
 */

extern int edg_wll_LogDone(edg_wll_Context context, const char * status_code, const char * reason, const int exit_code);


/**
 * \fn int edg_wll_LogDoneOK(edg_wll_Context context, const char * reason, const int exit_code); 
 * \brief simple wrapper around edg_wll_LogEvent for event Done, status_code OK
 * \param context	context to work with,
 * \param reason	reason for the change
 * \param exit_code	process exit code
 * \see edg_wll_LogEvent()
 */

extern int edg_wll_LogDoneOK(edg_wll_Context context, const char * reason, const int exit_code);


/**
 * \fn int edg_wll_LogDoneFAILED(edg_wll_Context context, const char * reason, const int exit_code); 
 * \brief simple wrapper around edg_wll_LogEvent for event Done, status_code FAILED
 * \param context	context to work with,
 * \param reason	reason for the change
 * \param exit_code	process exit code
 * \see edg_wll_LogEvent()
 */

extern int edg_wll_LogDoneFAILED(edg_wll_Context context, const char * reason, const int exit_code);


/**
 * \fn int edg_wll_LogDoneCANCELLED(edg_wll_Context context, const char * reason, const int exit_code); 
 * \brief simple wrapper around edg_wll_LogEvent for event Done, status_code CANCELLED
 * \param context	context to work with,
 * \param reason	reason for the change
 * \param exit_code	process exit code
 * \see edg_wll_LogEvent()
 */

extern int edg_wll_LogDoneCANCELLED(edg_wll_Context context, const char * reason, const int exit_code);


/**
 * \fn int edg_wll_LogCancel(edg_wll_Context context, const char * status_code, const char * reason); 
 * \brief simple wrapper around edg_wll_LogEvent for event Cancel
 * \param context	context to work with,
 * \param status_code	classification of the cancel
 * \param reason	detailed description
 * \see edg_wll_LogEvent()
 */

extern int edg_wll_LogCancel(edg_wll_Context context, const char * status_code, const char * reason);


/**
 * \fn int edg_wll_LogCancelREQ(edg_wll_Context context, const char * reason); 
 * \brief simple wrapper around edg_wll_LogEvent for event Cancel, status_code REQ
 * \param context	context to work with,
 * \param reason	detailed description
 * \see edg_wll_LogEvent()
 */

extern int edg_wll_LogCancelREQ(edg_wll_Context context, const char * reason);


/**
 * \fn int edg_wll_LogCancelREFUSE(edg_wll_Context context, const char * reason); 
 * \brief simple wrapper around edg_wll_LogEvent for event Cancel, status_code REFUSE
 * \param context	context to work with,
 * \param reason	detailed description
 * \see edg_wll_LogEvent()
 */

extern int edg_wll_LogCancelREFUSE(edg_wll_Context context, const char * reason);


/**
 * \fn int edg_wll_LogCancelDONE(edg_wll_Context context, const char * reason); 
 * \brief simple wrapper around edg_wll_LogEvent for event Cancel, status_code DONE
 * \param context	context to work with,
 * \param reason	detailed description
 * \see edg_wll_LogEvent()
 */

extern int edg_wll_LogCancelDONE(edg_wll_Context context, const char * reason);


/**
 * \fn int edg_wll_LogCancelABORT(edg_wll_Context context, const char * reason); 
 * \brief simple wrapper around edg_wll_LogEvent for event Cancel, status_code ABORT
 * \param context	context to work with,
 * \param reason	detailed description
 * \see edg_wll_LogEvent()
 */

extern int edg_wll_LogCancelABORT(edg_wll_Context context, const char * reason);


/**
 * \fn int edg_wll_LogAbort(edg_wll_Context context, const char * reason); 
 * \brief simple wrapper around edg_wll_LogEvent for event Abort
 * \param context	context to work with,
 * \param reason	reason of abort
 * \see edg_wll_LogEvent()
 */

extern int edg_wll_LogAbort(edg_wll_Context context, const char * reason);


/**
 * \fn int edg_wll_LogClear(edg_wll_Context context, const char * reason); 
 * \brief simple wrapper around edg_wll_LogEvent for event Clear
 * \param context	context to work with,
 * \param reason	why the job was cleared
 * \see edg_wll_LogEvent()
 */

extern int edg_wll_LogClear(edg_wll_Context context, const char * reason);


/**
 * \fn int edg_wll_LogClearUSER(edg_wll_Context context); 
 * \brief simple wrapper around edg_wll_LogEvent for event Clear, reason USER
 * \param context	context to work with,
 * \see edg_wll_LogEvent()
 */

extern int edg_wll_LogClearUSER(edg_wll_Context context);


/**
 * \fn int edg_wll_LogClearTIMEOUT(edg_wll_Context context); 
 * \brief simple wrapper around edg_wll_LogEvent for event Clear, reason TIMEOUT
 * \param context	context to work with,
 * \see edg_wll_LogEvent()
 */

extern int edg_wll_LogClearTIMEOUT(edg_wll_Context context);


/**
 * \fn int edg_wll_LogClearNOOUTPUT(edg_wll_Context context); 
 * \brief simple wrapper around edg_wll_LogEvent for event Clear, reason NOOUTPUT
 * \param context	context to work with,
 * \see edg_wll_LogEvent()
 */

extern int edg_wll_LogClearNOOUTPUT(edg_wll_Context context);


/**
 * \fn int edg_wll_LogPurge(edg_wll_Context context); 
 * \brief simple wrapper around edg_wll_LogEvent for event Purge
 * \param context	context to work with,
 * \see edg_wll_LogEvent()
 */

extern int edg_wll_LogPurge(edg_wll_Context context);


/**
 * \fn int edg_wll_LogMatch(edg_wll_Context context, const char * dest_id); 
 * \brief simple wrapper around edg_wll_LogEvent for event Match
 * \param context	context to work with,
 * \param dest_id	Id of the destination CE/queue
 * \see edg_wll_LogEvent()
 */

extern int edg_wll_LogMatch(edg_wll_Context context, const char * dest_id);


/**
 * \fn int edg_wll_LogPending(edg_wll_Context context, const char * reason); 
 * \brief simple wrapper around edg_wll_LogEvent for event Pending
 * \param context	context to work with,
 * \param reason	why matching CE cannot be found
 * \see edg_wll_LogEvent()
 */

extern int edg_wll_LogPending(edg_wll_Context context, const char * reason);


/**
 * \fn int edg_wll_LogRegJob(edg_wll_Context context, const char * jdl, const char * ns, const edg_wlc_JobId parent, const char * jobtype, const int nsubjobs, const char * seed); 
 * \brief simple wrapper around edg_wll_LogEvent for event RegJob
 * \param context	context to work with,
 * \param jdl	job description
 * \param ns	NetworkServer handling the job
 * \param parent	jobid of parent job
 * \param jobtype	job type
 * \param nsubjobs	number of subjobs
 * \param seed	seed for subjob id generation
 * \see edg_wll_LogEvent()
 */

extern int edg_wll_LogRegJob(edg_wll_Context context, const char * jdl, const char * ns, const edg_wlc_JobId parent, const char * jobtype, const int nsubjobs, const char * seed);


/**
 * \fn int edg_wll_LogRegJobSIMPLE(edg_wll_Context context, const char * jdl, const char * ns, const edg_wlc_JobId parent, const int nsubjobs, const char * seed); 
 * \brief simple wrapper around edg_wll_LogEvent for event RegJob, jobtype SIMPLE
 * \param context	context to work with,
 * \param jdl	job description
 * \param ns	NetworkServer handling the job
 * \param parent	jobid of parent job
 * \param nsubjobs	number of subjobs
 * \param seed	seed for subjob id generation
 * \see edg_wll_LogEvent()
 */

extern int edg_wll_LogRegJobSIMPLE(edg_wll_Context context, const char * jdl, const char * ns, const edg_wlc_JobId parent, const int nsubjobs, const char * seed);


/**
 * \fn int edg_wll_LogRegJobDAG(edg_wll_Context context, const char * jdl, const char * ns, const edg_wlc_JobId parent, const int nsubjobs, const char * seed); 
 * \brief simple wrapper around edg_wll_LogEvent for event RegJob, jobtype DAG
 * \param context	context to work with,
 * \param jdl	job description
 * \param ns	NetworkServer handling the job
 * \param parent	jobid of parent job
 * \param nsubjobs	number of subjobs
 * \param seed	seed for subjob id generation
 * \see edg_wll_LogEvent()
 */

extern int edg_wll_LogRegJobDAG(edg_wll_Context context, const char * jdl, const char * ns, const edg_wlc_JobId parent, const int nsubjobs, const char * seed);


/**
 * \fn int edg_wll_LogRegJobPARTITIONABLE(edg_wll_Context context, const char * jdl, const char * ns, const edg_wlc_JobId parent, const int nsubjobs, const char * seed); 
 * \brief simple wrapper around edg_wll_LogEvent for event RegJob, jobtype PARTITIONABLE
 * \param context	context to work with,
 * \param jdl	job description
 * \param ns	NetworkServer handling the job
 * \param parent	jobid of parent job
 * \param nsubjobs	number of subjobs
 * \param seed	seed for subjob id generation
 * \see edg_wll_LogEvent()
 */

extern int edg_wll_LogRegJobPARTITIONABLE(edg_wll_Context context, const char * jdl, const char * ns, const edg_wlc_JobId parent, const int nsubjobs, const char * seed);


/**
 * \fn int edg_wll_LogRegJobPARTITIONED(edg_wll_Context context, const char * jdl, const char * ns, const edg_wlc_JobId parent, const int nsubjobs, const char * seed); 
 * \brief simple wrapper around edg_wll_LogEvent for event RegJob, jobtype PARTITIONED
 * \param context	context to work with,
 * \param jdl	job description
 * \param ns	NetworkServer handling the job
 * \param parent	jobid of parent job
 * \param nsubjobs	number of subjobs
 * \param seed	seed for subjob id generation
 * \see edg_wll_LogEvent()
 */

extern int edg_wll_LogRegJobPARTITIONED(edg_wll_Context context, const char * jdl, const char * ns, const edg_wlc_JobId parent, const int nsubjobs, const char * seed);


/**
 * \fn int edg_wll_LogChkpt(edg_wll_Context context, const char * tag, const char * classad); 
 * \brief simple wrapper around edg_wll_LogEvent for event Chkpt
 * \param context	context to work with,
 * \param tag	checkpoint tag
 * \param classad	checkpoint value
 * \see edg_wll_LogEvent()
 */

extern int edg_wll_LogChkpt(edg_wll_Context context, const char * tag, const char * classad);


/**
 * \fn int edg_wll_LogListener(edg_wll_Context context, const char * svc_name, const char * svc_host, const uint16_t svc_port); 
 * \brief simple wrapper around edg_wll_LogEvent for event Listener
 * \param context	context to work with,
 * \param svc_name	port instance name
 * \param svc_host	hostname
 * \param svc_port	port number
 * \see edg_wll_LogEvent()
 */

extern int edg_wll_LogListener(edg_wll_Context context, const char * svc_name, const char * svc_host, const uint16_t svc_port);


/**
 * \fn int edg_wll_LogCurDescr(edg_wll_Context context, const char * descr); 
 * \brief simple wrapper around edg_wll_LogEvent for event CurDescr
 * \param context	context to work with,
 * \param descr	description of current job transformation (output of helper)
 * \see edg_wll_LogEvent()
 */

extern int edg_wll_LogCurDescr(edg_wll_Context context, const char * descr);


/**
 * \fn int edg_wll_LogUserTag(edg_wll_Context context, const char * name, const char * value); 
 * \brief simple wrapper around edg_wll_LogEvent for event UserTag
 * \param context	context to work with,
 * \param name	tag name
 * \param value	tag value
 * \see edg_wll_LogEvent()
 */

extern int edg_wll_LogUserTag(edg_wll_Context context, const char * name, const char * value);


/**
 * \fn int edg_wll_LogChangeACL(edg_wll_Context context, const char * user_id, const int user_id_type, const int permission, const int permission_type, const int operation); 
 * \brief simple wrapper around edg_wll_LogEvent for event ChangeACL
 * \param context	context to work with,
 * \param user_id	DN or VOMS parameter (in format VO:group)
 * \param user_id_type	type of information given in user_id (DN or VOMS)
 * \param permission	ACL permission to change (currently only READ)
 * \param permission_type	type of permission requested ('allow', 'deny')
 * \param operation	operation requested to perform with ACL (add, remove)
 * \see edg_wll_LogEvent()
 */

extern int edg_wll_LogChangeACL(edg_wll_Context context, const char * user_id, const int user_id_type, const int permission, const int permission_type, const int operation);



/**
 * Formats a logging message and sends it asynchronously to local-logger
 * \brief generic asynchronous logging function
 * \param context	INOUT context to work with,
 * \param event		IN type of the event,
 * \param fmt		IN printf()-like format string,
 * \param ...		IN event specific values/data according to fmt,
 * \retval 0		successful completition,
 * \retval EINVAL	bad jobId, unknown event code, or the format string together with the remaining arguments does not form a valid event,
 * \retval ENOSPC	L&B infrastructure failed to accept the event due to lack of disk space etc.,
 * \retval ENOMEM	failed to allocate memory,
 * \retval ECONNREFUSED	cannot connect to the specified local logger,
 * \retval EAGAIN	non blocking return from the call, the event may or may not get logged,
 * \retval EDG_WLL_ERROR_NOJOBID logging call attempted without assigning jobId to the context.
 */
extern int edg_wll_LogEvent(
	edg_wll_Context context,
	edg_wll_EventCode event,
	char *fmt, ...);


/**
 * Formats a logging message and sends it synchronously to local-logger
 * \brief generic synchronous logging function
 * \param context	INOUT context to work with,
 * \param event		IN type of the event,
 * \param fmt		IN printf()-like format string,
 * \param ...		IN event specific values/data according to fmt,
 * \retval 0		successful completition,
 * \retval EINVAL	bad jobId, unknown event code, or the format string together with the remaining arguments does not form a valid event,
 * \retval ENOSPC	L&B infrastructure failed to accept the event due to lack of disk space etc.,
 * \retval ENOMEM	failed to allocate memory,
 * \retval ECONNREFUSED	cannot connect to the specified local logger,
 * \retval EAGAIN	non blocking return from the call, the event may or may not get logged,
 * \retval EDG_WLL_ERROR_NOJOBID logging call attempted without assigning jobId to the context,
 * \retval EPERM	the user is not authorized to add events to this job,
 * \retval EDG_WLL_ERROR_DB_DUP_KEY exactly the same event has been already stored.
 */
extern int edg_wll_LogEventSync(
	edg_wll_Context context,
	edg_wll_EventCode event,
	char *fmt, ...);


/**
 * Instructs interlogger to to deliver all pending events related to current job
 * \brief flush events from interlogger
 * \note sort of status query more than a command
 * \param context	INOUT context to work with,
 * \param timeout	INOUT wait at most this much time for completition, remaining time on return,
 * \retval 0		successful completition,
 * \retval EDG_WLL_ERROR_INTERLOG_TIMEOUT the inter-logger did not respond within the timeout,
 * \retval EDG_WLL_ERROR_INTERLOG_CONLOST inter-logger lost connection to one or more servers,
 * \retval EDG_WLL_ERROR_INTERLOG_AGAIN   not all pending events were delivered within the timeout.
 */
extern int edg_wll_LogFlush(
	edg_wll_Context context,
	struct timeval *timeout);


/**
 * Instructs interlogger to to deliver all pending events
 * \brief flush all events from interlogger
 * \note same as edg_wll_LogFlush() for all jobs known to interlogger
 * \see edg_wll_LogFlush()
 */
extern int edg_wll_LogFlushAll(
	edg_wll_Context context,
	struct timeval *timeout);

/**
 * Set a current job for given context.
 * \note Should be called before any logging call.
 * \param context INOUT context to work with
 * \param job IN further logging calls are related to this job
 * \param code IN sequence code as obtained from previous component
 * \param flags IN flags on code handling (\see API documentation)
 */
extern int edg_wll_SetLoggingJob(
	edg_wll_Context	context,
	const edg_wlc_JobId	job,
	const char *		code,
	int			flags
);


/**
 * Register job with L&B service.
 * Done via logging REGJOB event, may generate subjob id's and create
 * the parent-children associations.
 * Set the job as current for the context and initialize sequence code.
 *
 * Partitionable jobs should set num_subjobs=0 initially,
 * and re-register when number of subjobs becomes known.
 *
 * \param type EDG_WLL_JOB_SIMPLE,  EDG_WLL_JOB_DAG, or EDG_WLL_JOB_PARTITIONABLE
 * \param jdl user-specified JDL
 * \param ns network server contact
 * \param num_subjobs number of subjobs to create
 * \param seed seed used for subjob id's generator.
 * 	Use non-NULL value to be able to regenerate the set of jobid's
 * \param subjobs returned subjob id's
 */

/* backward compatibility */
#define EDG_WLL_JOB_SIMPLE	EDG_WLL_REGJOB_SIMPLE
 
extern int edg_wll_RegisterJob(
	edg_wll_Context		context,
	const edg_wlc_JobId	job,
	enum edg_wll_RegJobJobtype	type,
	const char *		jdl,
	const char *		ns,
	int			num_subjobs,
	const char *		seed,
	edg_wlc_JobId **	subjobs
);

/** 
 * Synchronous variant of edg_wll_RegisterJob
 */

extern int edg_wll_RegisterJobSync(
	edg_wll_Context		context,
	const edg_wlc_JobId	job,
	enum edg_wll_RegJobJobtype	type,
	const char *		jdl,
	const char *		ns,
	int			num_subjobs,
	const char *		seed,
	edg_wlc_JobId **	subjobs
);

/**
 * Register subjobs in a batch.
 * Mainly used to provide JDL's of individual subjobs in a more efficient
 * way than logging them one by one.
 * \param jdls	array of JDL's
 * \param subjobs 	array of jobid's in the same order
 */

extern int edg_wll_RegisterSubjobs(
	edg_wll_Context		context,
	const edg_wlc_JobId	parent,
	char const * const *	jdls,
	const char *		ns,
	edg_wlc_JobId const *	subjobs
);


/**
 * Generate or regenerate set of subjob ID's.
 * Calls the same algorithm used to generate subjob ID's in edg_wll_RegisterJob().
 * Local semantics only, server is not contacted.
 */

extern int edg_wll_GenerateSubjobIds(
	edg_wll_Context		context,
	const edg_wlc_JobId	parent,
	int			num_subjobs,
	const char *		seed,
	edg_wlc_JobId **	subjobs
);


enum edg_wll_Permission {
	EDG_WLL_PERM_READ  = 1,
	EDG_WLL_PERM_WRITE = 4,
	EDG_WLL_PERM_ADMIN = 8,
};

enum edg_wll_PermissionType {
	EDG_WLL_PERM_ALLOW,
	EDG_WLL_PERM_DENY,
};

enum edg_wll_ACLOperation {
	EDG_WLL_ACL_ADD,
	EDG_WLL_ACL_REMOVE,
};

enum edg_wll_UserIdType {
	EDG_WLL_USER_SUBJECT,		/* X.509 subject name */
	EDG_WLL_USER_VOMS_GROUP,	/* VOMS group membership */
};

/**
 * Change ACL for given job.
 * \param specification of user's credential
 * \param user_id_type type of user_id,
 *    for EDG_WLL_USER_SUBJECT the user_id parameter is expected to be user's subject name
 *    for EDG_WLL_USER_VOMS_GROUP the user_id is expected to be of the form VO:group specifying required group membersip as managed by VOMS
 * \param permission ACL permission to change
 * \param permission_type type of given permission (allow or deny operation) 
 * \param operation operation to perform with ACL (add or remove record)
 */
 
extern int edg_wll_ChangeACL(
	edg_wll_Context		context,
	const edg_wlc_JobId	job,
	const char *		user_id,
	enum edg_wll_UserIdType	user_id_type,
	enum edg_wll_Permission		permission,
	enum edg_wll_PermissionType	permission_type,
	enum edg_wll_ACLOperation	operation
);


#ifdef __cplusplus
}
#endif

#endif /* __EDG_WORKLOAD_LOGGING_CLIENT_PRODUCER_H__ */
